#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <ogcsys.h>
#include <gccore.h>
#include <wiiuse/wpad.h>

static u32 *xfb;
static GXRModeObj *rmode;

//Funciones de inicializacion
void Initialise() {

	VIDEO_Init();
	WPAD_Init(); //Se inicializa la librera wpad


	rmode = VIDEO_GetPreferredMode(NULL);

	xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	console_init(xfb,0,0,rmode->fbWidth,rmode->xfbHeight,rmode->fbWidth*VI_DISPLAY_PIX_SZ);

	VIDEO_Configure(rmode);
	VIDEO_SetNextFramebuffer(xfb);
	VIDEO_SetBlack(FALSE);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();


}

//Estas funciones dibujan un punto en la pantalla. Fuente: <http://www.codemii.com/2008/08/31/tutorial-4-cursors/>
void DrawHLine (int x1, int x2, int y, int color) {
	int i;
	y = 320 * y;
	x1 >>= 1;
	x2 >>= 1;
	for (i = x1; i <= x2; i++) {
		u32 *tmpfb = xfb;
		tmpfb[y+i] = color;
	}
}

void DrawVLine (int x, int y1, int y2, int color) {
	int i;
	x >>= 1;
	for (i = y1; i <= y2; i++) {
		u32 *tmpfb = xfb;
		tmpfb[x + (640 * i) / 2] = color;
	}
}

void DrawBox (int x1, int y1, int x2, int y2, int color) {
	DrawHLine (x1, x2, y1, color);
	DrawHLine (x1, x2, y2, color);
	DrawVLine (x1, y1, y2, color);
	DrawVLine (x2, y1, y2, color);
}


int main() {

	Initialise();

    //Se definen las estructuras que almacenarn los datos obtenidos del Wiimote
    ir_t ir;//infrarrojos
    vec3w_t vector;//vector de aceleracin
    orient_t orient;//orientacin

    u32 pressed;//Esta variable se usar para detectar si se mantiene pulsado un botn
    expansion_t exp;//Estructura para la expansin del Wiimote

    WPAD_SetDataFormat(0, WPAD_FMT_BTNS_ACC_IR); //Mediante la funcin WPAD_SetDataFormat se selecciona el modo de funcionamiento del Wiimote. En este caso se establece el modo de funcionamiento con infrarrojos
	WPAD_SetVRes(0, rmode->fbWidth, rmode->xfbHeight); //Se limita la resolucin de los infrarrojos a la resolucin de pantalla


	while(1) {

		WPAD_ScanPads();//WPAD_ScanPads obtiene los datos del Wiimote en cada bucle
		WPAD_IR(0,&ir);//Se almacenan los datos de los infrarrojos en la estructura ir_t
		WPAD_Accel(0, &vector);//Se almacenan los datos de la aceleracin en la estructura vec3w_t
		WPAD_Orientation(0, &orient);//Se almacenan los datos de la orientacin en la estructura orient_t

		// IR Movement
		pressed = WPAD_ButtonsHeld(0);//Se comprueba si se mantiene pulsado algn boton, y el resultado se almacena en la variable pressed


        console_init(xfb,0,0,rmode->fbWidth,rmode->xfbHeight,rmode->fbWidth*VI_DISPLAY_PIX_SZ);//Se reinicializa la consola para volver a imprimir los datos en cada bucle



		printf("\n\n\n");

		if(ir.valid) {//Se comprueba si el mando apunta a la pantalla
				printf(" Cursor: %.02f,%.02f\n",ir.x, ir.y);//Se imprimen las coordenadas del lugar a donde apunta el mando
				printf(" Angulo: %.02f grados\n",ir.angle);//ngulo de rotacin del mando
				printf(" Distancia: %.02fm\n", ir.z);//Distancia del mando a la pantalla
			} else {
				printf(" Sin Cursor\n\n");//En caso de que el mando no apunte a la pantalla, se imprime una aviso
			}
		printf(" Aceleracion:\n");
			printf(" XYZ: %3d,%3d,%3d\n",vector.x,vector.y,vector.z);//Vector de aceleracin tridimensional
			printf(" Pitch: %.02f\n",orient.pitch);//ngulo de 'pitch' del mando
			printf(" Roll: %.02f\n",orient.roll);//ngulo de 'roll' del mando. Equivale a la rotacin determinada por los infrarrojos.
			printf(" Yaw: %.02f\n",orient.yaw);//ngulo de 'yaw' del mando

		DrawBox (ir.x, ir.y, ir.x + 1, ir.y + 1, COLOR_WHITE);//Se dibuja un punto blanco en el lugar a donde apunte el mando

		printf(" Butones pulsados:\n ");//Se comprueba qu botn se est pulsando y se imprime su nombre
			if(pressed & WPAD_BUTTON_A) printf("A ");
			if(pressed & WPAD_BUTTON_B) printf("B ");
			if(pressed & WPAD_BUTTON_1) printf("1 ");
			if(pressed & WPAD_BUTTON_2) printf("2 ");
			if(pressed & WPAD_BUTTON_MINUS) printf("MINUS ");
			if(pressed & WPAD_BUTTON_HOME) printf("HOME ");
			if(pressed & WPAD_BUTTON_PLUS) printf("PLUS ");
			printf("\n ");
			if(pressed & WPAD_BUTTON_LEFT) printf("IZQUIERDA ");
			if(pressed & WPAD_BUTTON_RIGHT) printf("DERECHA ");
			if(pressed & WPAD_BUTTON_UP) printf("ARRIBA ");
			if(pressed & WPAD_BUTTON_DOWN) printf("ABAJO ");
			printf("\n");

        printf("Expansion:\n");


        WPAD_Expansion(0,&exp);//Se obtienen los datos de la expansin del Wiimote y se almacenan en la estructura expansion_t
        if(exp.type == EXP_NUNCHUK){//Si la expansin es un Nunchuk, se imprimen sus datos
            printf("Nunchuk\n");
            printf(" Butones pulsados:\n ");//Se comprueba que botn del nunchuk se pulsa y se imprime su nombre
            if(exp.nunchuk.btns_held & NUNCHUK_BUTTON_C){
                printf("Boton C pulsado.\n");
            }
            if(exp.nunchuk.btns_held & NUNCHUK_BUTTON_Z){
                printf("Boton Z pulsado.\n");
            }
            printf("Joystick: %.02f\n",exp.nunchuk.js.ang);//ngulo de la palanca del joystick
        }
        if(exp.type == EXP_NONE){
            printf("Ninguna\n");//Si no hay ninguna expansin se imprime este mensaje
        }


		if (pressed & WPAD_BUTTON_HOME) {//Al pulsar el botn 'Home' finaliza el programa
			exit(0);
		}


        VIDEO_Flush();
		VIDEO_WaitVSync();
	}

	return 0;
}
